﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.MetaOptimization {
  public class ParameterCombinationsEnumerator : IEnumerator<IOptimizable> {
    private IOptimizable node;
    private List<IEnumerator> enumerators;
    IEnumerator valueEnumerator;

    public ParameterCombinationsEnumerator(IOptimizable node) {
      this.node = node;
      this.enumerators = new List<IEnumerator>();
    }

    public IOptimizable Current {
      get { return node; }
    }
    object IEnumerator.Current {
      get { return Current; }
    }

    public void Dispose() { }

    public bool MoveNext() {
      int i = 0;
      bool ok = false;
      while (!ok && i < enumerators.Count) {
        if (enumerators[i].MoveNext()) {
          ok = true;
        } else {
          i++;
        }
      }

      if (ok) {
        for (int k = i - 1; k >= 0; k--) {
          enumerators[k].Reset();
          enumerators[k].MoveNext();
        }
      } else {
        return false;
      }

      var pc = node as IParameterConfiguration;
      if (pc != null && valueEnumerator != null) {
        pc.ActualValue = ((IValueConfiguration)valueEnumerator.Current).ActualValue;
        pc.UpdateActualValueIndexToItem((IValueConfiguration)valueEnumerator.Current);
      }
      var vc = node as IValueConfiguration;
      if (vc != null && valueEnumerator != null) {
        vc.ActualValue.Value = (IItem)valueEnumerator.Current;
      }
      return true;
    }

    public void Reset() {
      enumerators.Clear();
      valueEnumerator = null;

      var pc = node as IParameterConfiguration;
      if (pc != null) {
        valueEnumerator = pc.ValueConfigurations.CheckedItems.ToArray().GetEnumerator();
        //valueEnumerator.Reset();
        enumerators.Add(valueEnumerator);

        foreach (var valueConfiguration in pc.ValueConfigurations.CheckedItems) {
          if (valueConfiguration.Optimize) {
            var enumerator = new ParameterCombinationsEnumerator(valueConfiguration);
            enumerator.Reset();
            enumerator.MoveNext();
            enumerators.Add(enumerator);
          }
        }
      }

      var vc = node as IValueConfiguration;
      if (vc != null) {
        if (vc.RangeConstraint != null) {
          valueEnumerator = vc.RangeConstraint.GetCombinations().ToArray().GetEnumerator();
          //valueEnumerator.Reset();
          //enumerator.MoveNext();
          enumerators.Add(valueEnumerator);
        } else {
          foreach (var parameterConfiguration in vc.ParameterConfigurations) {
            if (parameterConfiguration.Optimize) {
              var enumerator = new ParameterCombinationsEnumerator(parameterConfiguration);
              enumerator.Reset();
              enumerator.MoveNext();
              enumerators.Add(enumerator);
            }
          }
        }
      }
    }

  }
}
